<?php
namespace Teknisa\Model\Teknisa\Repositories;

class Patchverprod extends \Doctrine\ORM\EntityRepository {

    public function getPatchVersionProduct($cdCliente, $cdProdGrup){
        $query = <<<SQL
        SELECT SUBSTR(TO_CHAR(TO_NUMBER(MAX(PP.CDVERPRODUTO||PP.CDRELVERPROD||PP.CDPATCHVERPR))-1),1,1)||'.'||
               SUBSTR(TO_CHAR(TO_NUMBER(MAX(PP.CDVERPRODUTO||PP.CDRELVERPROD||PP.CDPATCHVERPR))-1),2,2)||'.'||
               SUBSTR(TO_CHAR(TO_NUMBER(MAX(PP.CDVERPRODUTO||PP.CDRELVERPROD||PP.CDPATCHVERPR))-1),4,3) CDPATCHVERPR
          FROM PATCHVERPROD PP, VERSAOPROD VP, VERSAODISP VD, VERSAODISPCLI VDC
         WHERE (PP.CDPRODGRUP   = :P_CDPRODGRUP)
           AND (VP.CDPRODGRUP   = PP.CDPRODGRUP)
           AND (VP.CDVERPRODUTO = PP.CDVERPRODUTO)
           AND (VP.IDVERSAOATIV = 'S')
           AND (VP.IDVERSAOLIBE = 'S')
           AND (UPPER(SUBSTR(PP.CDVERPRODUTO,1,1)) <> 'V')
           AND PP.CDVERPRODUTO  = VD.CDVERSAODISP
           AND PP.CDRELVERPROD  = VD.CDRELVERSAODISP
           AND VDC.CDCLIENTE    = :P_CDCLIENTE
           AND VDC.NRVERSAODISP = VD.NRVERSAODISP
SQL;
        $params = array(
            'P_CDPRODGRUP' => $cdProdGrup,
            'P_CDCLIENTE' => $cdCliente
        );
        $patchVersionProduct = $this->getEntityManager()->getConnection()->fetchAll($query, $params);

        if (empty($patchVersionProduct) || ($patchVersionProduct && $patchVersionProduct[0]['CDPATCHVERPR'] == "..")){
            return null;
        } else {
            return $patchVersionProduct[0]['CDPATCHVERPR'];
        }
    }

    public function getProdGrup($mainVersion, $releaseVersion, $patchVersion){
        $query = <<<SQL
		SELECT CDPRODGRUP
          FROM PATCHVERPROD
         WHERE (CDVERPRODUTO = :P_CDVERPRODUTO)
           AND (CDRELVERPROD = :P_CDRELVERPROD)
           AND (CDPATCHVERPR = :P_CDPATCHVERPR)
SQL;
        $params = array(
            'P_CDVERPRODUTO' => $mainVersion,
            'P_CDRELVERPROD' => $releaseVersion,
            'P_CDPATCHVERPR' => $patchVersion
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getDatabaseVersion($codProdGrup, $mainVersion, $releaseVersion, $patchVersion){
        $query = <<<SQL
		SELECT NRVEROBJBD
          FROM PATCHVERPROD
         WHERE (CDVERPRODUTO = :P_CDVERPRODUTO)
           AND (CDRELVERPROD = :P_CDRELVERPROD)
           AND (CDPATCHVERPR = :P_CDPATCHVERPR)
           AND (CDPRODGRUP = :P_CDPRODGRUP)
SQL;
        $params = array(
            'P_CDVERPRODUTO' => $mainVersion,
            'P_CDRELVERPROD' => $releaseVersion,
            'P_CDPATCHVERPR' => $patchVersion,
            'P_CDPRODGRUP' => $codProdGrup
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getProductsAvailables($codArvProd, $nivelProd){
        $query = <<<SQL
        SELECT PP.CDARVPROD, PP.NMPRODUTO, PP.NMPRODINGLES, PP.NMPRODESPANH, PP.CDPRODUTO
          FROM PRODUTO PP, VERSAOPROD VP
         WHERE (PP.CDNVPRODUTO            = :P_CDNVPROD)
           AND (SUBSTR(PP.CDARVPROD,1,2) >= :P_ARVPROD)
           AND (PP.CDPRODUTO              = VP.CDPRODGRUP)
           AND (VP.IDVERSAOATIV           = 'S')
           AND (VP.IDVERSAOLIBE           = 'S')
         ORDER BY PP.CDARVPROD
SQL;

        $params = array(
            'P_CDNVPROD' => $nivelProd,
            'P_ARVPROD'  => $codArvProd
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }


    public function getSOlicitationAuditProductsAvailables($codArvProd, $nivelProd){
        $query = <<<SQL
        SELECT PP.CDARVPROD, PP.NMPRODUTO, PP.NMPRODINGLES, PP.NMPRODESPANH, PP.CDPRODUTO
          FROM PRODUTO PP, VERSAOPROD VP
         WHERE (PP.CDNVPRODUTO            = :P_CDNVPROD)
           AND (SUBSTR(PP.CDARVPROD,1,2) >= :P_ARVPROD)
           AND (PP.CDPRODUTO              = VP.CDPRODGRUP)
           AND (VP.IDVERSAOATIV           = 'S')
           AND (VP.IDVERSAOLIBE           = 'S')
         ORDER BY PP.CDARVPROD
SQL;

        $params = array(
            'P_CDNVPROD' => $nivelProd,
            'P_ARVPROD'  => $codArvProd
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getDescriptionByRequest($requestId){
         $query = <<<SQL
        SELECT DSIMPLSOLI
        FROM IMPLSOLI
        WHERE NRSOLISERV = :REQUEST_ID
SQL;
        $params = array(
            "REQUEST_ID" => $requestId
        );

        $description = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return empty($description) ? "" : $description[0]['DSIMPLSOLI'];
    }

    public function getRequestsAnswered($initialPatch, $endPatch, $cdCliente = 'T', $conditions = ''){
        $query = <<<SQL
        SELECT *
          FROM (SELECT PRO.CDARVPROD, PRO.NMPRODUTO, SPP.NRSOLISERV, SOL.DSREFESOLI,IMP.DSIMPLSOLI AS REQUEST_DESCRIPTION,DES.DSSOLISERV,
                       SPP.CDVERPRODUTO||'.'||SPP.CDRELVERPROD||'.'||SPP.CDPATCHVERPR AS CDVERSAO,
                       SOL.CDCLIENTE, TO_CHAR(SPP.DTLIBPATVPRO, 'DD/MM/YYYY') AS DTLIBPATVPRO,PRO1.NMPRODUTO AS PRODUCT
                  FROM PRODUTO PRO, SOLIPATCHPRO SPP, SOLISERV SOL,IMPLSOLI IMP,DESCSOLI DES,PRODUTO PRO1
                 WHERE SOL.NRSOLISERV = SPP.NRSOLISERV
                   AND SPP.CDPRODGRUP = PRO.CDPRODUTO
                   AND IMP.NRSOLISERV(+) = SOL.NRSOLISERV
                   AND DES.NRSOLISERV = SOL.NRSOLISERV
                   AND PRO1.CDPRODUTO = SOL.CDPRODCOMP
                   AND (SPP.CDVERPRODUTO||SPP.CDRELVERPROD||SPP.CDPATCHVERPR > :P_PATCHINI)
                   AND (SPP.CDVERPRODUTO||SPP.CDRELVERPROD||SPP.CDPATCHVERPR <= :P_PATCHFIN)
                   AND ((SOL.CDCLIENTE = :P_CDCLIENTE) OR ('T' = :P_CDCLIENTE))

              ORDER BY PRO.CDPRODUTO, SPP.CDVERPRODUTO, SPP.CDRELVERPROD,SPP.CDPATCHVERPR, SPP.NRSOLISERV)
         WHERE 1 = 1
               $conditions
SQL;
        $params = array(
            "P_CDCLIENTE" => $cdCliente,
            "P_PATCHINI" => $initialPatch,
            "P_PATCHFIN" => $endPatch
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    public function getSolicitationAudit($initialPatch, $endPatch, $isCurrentClient, $cdCliente) {
        $customerQueryOperator = $isCurrentClient ? '=' : '<>';
        $query = "SELECT CP.NRCALL, S.*
                   FROM ( SELECT PRO.CDARVPROD, PRO.NMPRODUTO, SPP.NRSOLISERV, SOL.DSREFESOLI,IMP.DSIMPLSOLI AS REQUEST_DESCRIPTION,DES.DSSOLISERV,
                                 SPP.CDVERPRODUTO||'.'||SPP.CDRELVERPROD||'.'||SPP.CDPATCHVERPR AS CDVERSAO,
                                 SOL.CDCLIENTE, TO_CHAR(SPP.DTLIBPATVPRO, 'DD/MM/YYYY') AS DTLIBPATVPRO,PRO1.NMPRODUTO AS PRODUCT
                            FROM PRODUTO PRO, SOLIPATCHPRO SPP, SOLISERV SOL,IMPLSOLI IMP,DESCSOLI DES,PRODUTO PRO1
                           WHERE SOL.NRSOLISERV = SPP.NRSOLISERV
                             AND SPP.CDPRODGRUP = PRO.CDPRODUTO
                             AND IMP.NRSOLISERV = SOL.NRSOLISERV
                             AND DES.NRSOLISERV = SOL.NRSOLISERV
                             AND PRO1.CDPRODUTO = SOL.CDPRODCOMP
                             AND (SPP.CDVERPRODUTO||SPP.CDRELVERPROD||SPP.CDPATCHVERPR > :P_PATCHINI)
                             AND (SPP.CDVERPRODUTO||SPP.CDRELVERPROD||SPP.CDPATCHVERPR <= :P_PATCHFIN)
                             AND (SOL.CDCLIENTE $customerQueryOperator :P_CDCLIENTE)
                       ORDER BY PRO.CDPRODUTO, SPP.CDVERPRODUTO, SPP.CDRELVERPROD,SPP.CDPATCHVERPR, SPP.NRSOLISERV
                        ) S
               LEFT JOIN CP_CALL CP ON (CP.CALL_RELATIONSHIP = S.NRSOLISERV AND CP.TYPE = 'S')";

        $params = array(
            "P_CDCLIENTE" => $cdCliente,
            "P_PATCHINI"  => $initialPatch,
            "P_PATCHFIN"  => $endPatch
        );

        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }
}
